@@ -1,4 +1,5 @@
-#ifndef SYSTEM_INCLUDED
+#ifndef __SYSTEM_INCLUDED__
+#define	__SYSTEM_INCLUDED__
 
 /*
 	=================================
@@ -13,6 +14,10 @@ typedef unsigned char		UBYTE;
 typedef unsigned short int	UWORD;
 typedef unsigned long int	ULONG;
 
-#define	SYSTEM_INCLUDED
+#define	CURSES_LEFT	0x00
+#define	CURSES_CENTRAL	0x01
+#define	CURSES_RIGHT	0x02
+#define	CURSES_WIDE_1	0x03
+#define	CURSES_WIDE_2	0x04
 
 #endif
